#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""@package docstring
This test runs the four different cases as defined in table 2 of the paper
with a SIMPLE-GS-GS preconditioner.
"""





import katoptron.CMAME.model as model

import katoptron as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katoptron.utilities as u


def evaluate_one_ensemble(ensemble_size, case_id, first_sample_id, msh, comm,
                          file_dir):
    """
    This function evaluates one ensemble for a given test case
    """
    work_dir = os.getcwd()
    args = parseargs()

    p = model.get_parameters(case_id, first_sample_id)
    p['MueLu xml template file name'] = '/../../preconditioners/SIMPLE_gs_direct_lvl_MT.xml.in'

    pm = model.plate_model(ensemble_size, np.array([0]), msh, comm, file_dir,
                           work_dir, p)
    pm.run()


def evaluate_all_ensembles(ensemble_sizes, case_ids, N, msh, comm, file_dir):
    """
    This function loop over the test cases, the ensemble sizes,
    and the ensembles
    """
    for case_id in case_ids:
        directory_case_id = 'case_' + str(case_id)
        if not os.path.exists(directory_case_id):
            os.makedirs(directory_case_id)
        os.chdir(directory_case_id)
        for ensemble_size in ensemble_sizes:
            directory = str(ensemble_size)
            if not os.path.exists(directory):
                os.makedirs(directory)
            os.chdir(directory)

            N_ensemble = N // ensemble_size
            for ensemble_index in range(0, N_ensemble):
                directory = str(ensemble_index)
                if not os.path.exists(directory):
                    os.makedirs(directory)
                os.chdir(directory)

                first_sample_id = ensemble_index * ensemble_size

                evaluate_one_ensemble(ensemble_size, case_id, first_sample_id,
                                      msh, comm, file_dir)
                os.chdir('..')
            os.chdir('..')
        os.chdir('..')


def main():
    """
    This loads the mesh and define the case ids and ensemble sizes
    """
    comm, rank, size = m.utilities.import_MPI()

    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.geo', 'plate_1_tmp', 'plate_2.geo',
                        'plate_2_tmp', 'plate_fused', file_dir + '/../mesh',
                        work_dir, comm, rank, size)

    deterministic = False

    if deterministic:
        N = 1
        ensemble_sizes = [1]
        case_ids = [0]
    else:
        N = 96
        ensemble_sizes = [1, 8, 16, 24, 32]
        case_ids = [1, 2, 3, 4, 5, 6, 7, 8, 9]

    nThreads = u.Initialize_Kokkos()
    evaluate_all_ensembles(ensemble_sizes, case_ids, N, msh, comm, file_dir)
    u.Finalize_Kokkos()


if __name__ == "__main__":
    main()
